﻿using CMS.Base;
using CMS.DataEngine;
using CMS.DocumentEngine;
using CMS.Helpers;
using CMS.PortalEngine;
using CMS.WorkflowEngine;

[LogChangeHandler]
public partial class CMSModuleLoader
{
    /// <summary>
    /// Custom attribute class.
    /// </summary>
    private class LogChangeHandler : CMSLoaderAttribute
    {
        /// <summary>
        /// Called automatically when the application starts
        /// </summary>
        public override void Init()
        {
            // Assigns a handler to the DocumentEvents.LogChange.Before event
            // This event occurs when pages are modified, before the system starts logging the changes
            DocumentEvents.LogChange.Before += LogDocumentChange_Before;

            // Assigns a handler to the LogChange.Before event for PageTemplateInfo objects
            // This event occurs when page templates are modified, before the system starts logging the changes
            PageTemplateInfo.TYPEINFO.Events.LogChange.Before += LogPageTemplateObjectChange_Before;
        }

        // Disables logging of staging and outgoing integration tasks for the content of pages in the /Archive section of the website
        private void LogDocumentChange_Before(object sender, LogDocumentChangeEventArgs e)
        {
            #region Exclude Development
            // Gets the synchronized page
            TreeNode page = e.Settings.Node;

            // Excludes pages under the "/Contact" path from content staging and integration bus synchronization
            if (page.NodeAliasPath.StartsWithCSafe("/Development"))
            {
                e.Using(new CMSActionContext()
                {
                    LogSynchronization = false,
                    LogIntegration = false
                }
                );
            }
            #endregion

            #region AutoPublishing
            //Check if the document is under an "auto-publishing" workflow. If so, do not log the PublishDocument event to prevent redudant staging tasks.
            WorkflowInfo winfo = e.Settings.Node.GetWorkflow();
            if (winfo != null)
            {
                //Make sure the page is in the correct workflow
                if (winfo.WorkflowName.ToLower() == "sampleworkflowwithsychronization")
                {
                    //Check if this is the final publish step
                    if (page.WorkflowStepName.ToLower() == "published")
                    {
                        e.Settings.LogStaging = false;
                    }
                }
            }
            #endregion
        }

        // Disables 'Create' and 'Update' staging/integration tasks for ad-hoc page templates of pages in the excluded /Archive section
        private void LogPageTemplateObjectChange_Before(object sender, LogObjectChangeEventArgs e)
        {
            // Gets the synchronized object
            GeneralizedInfo obj = e.Settings.InfoObj;

            if (obj != null)
            {
                // Continues only for ad-hoc templates
                if (!ValidationHelper.GetBoolean(obj.GetValue("PageTemplateIsReusable"), false))
                {
                    // Checks if the template is used by a page in the /Archive section
                    var nodeQuery = DocumentHelper.GetDocuments()
                                                    .Path("/Development", PathTypeEnum.Section)
                                                    .WhereEquals("NodeTemplateID", obj.GetValue("PageTemplateID"));

                    if (nodeQuery.Count > 0)
                    {
                        // Disables logging of staging and integration tasks
                        e.Settings.LogStaging = false;
                        e.Settings.LogIntegration = false;
                    }
                }
            }
        }
    }
}